//+------------------------------------------------------------------+
//|                                                      ASorter.mqh |
//|                                  2019 - 2021, dimitri pecheritsa |
//|                             https://www.mql5.com/en/users/dmipec |
//+------------------------------------------------------------------+
//+------------------------------------------------------------------+
//| template abstract sorter class                                   |
//|  this is a common class for various descending sorting algorithms|
//|  sort - is a pure virtual method and must be defined by children |
//|classes.                                                          |
//|  keys - is the main array which contains values of any type that |
//|must be sorted.                                                   |
//|  items - can be of the same type as the keys array, but it does  |
//|not have to. this array mimics the swapping procedures executed   |
//|with the keys array, however it doesn't participate in the        |
//|comparisons or the algorithm steps. as a result the items array   |
//|will get sorted by the elemets of the keys array. this may come   |
//|handy in variuos applications.                                    |
//+------------------------------------------------------------------+
template<typename TKey,typename TItem>
class ASorter
  {
public:
   TKey              keys[];
   TItem             items[];
                     ASorter(void) {}
                    ~ASorter(void) {}
   virtual void      Sort(void)=0;
protected:
   virtual void      Swap(int a,int b);
  };
//+------------------------------------------------------------------+
//| swap                                                             |
//|  a default implementation of a virtual swapping method of keys   |
//|(and items) at indexes a and b, applicable to most concrete       |
//|algorithms, where it can be used as is. however, children may     |
//|redefine this method, if they decide that it is unfit for their   |
//|use, or just ignore this method altogether.                       |
//|  the items array participates in swapping if its length matches  |
//|the length of the keys, otherwise it is ignored, thus only the    |
//|keys will eventually get sorted                                   |
//+------------------------------------------------------------------+
template<typename TKey,typename TItem>
void ASorter::Swap(int a,int b)
  {
//--- swap keys
   TKey key=keys[a];
   keys[a]=keys[b];
   keys[b]=key;
//--- check items
   if(ArraySize(items)!=ArraySize(keys))
      return;
//--- swap items
   TItem item=items[a];
   items[a]=items[b];
   items[b]=item;
  }
//+------------------------------------------------------------------+
